Package: vmrseq
Type: Package
Title: Probabilistic Modeling of Single-cell Methylation Heterogeneity
Version: 1.5.1
Authors@R: 
     c(person("Keegan", "Korthauer", , "kkorthauer@cmmt.ubc.ca", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-4565-1654")),
      person("Ning", "Shen", , "ning.shen.wk@gmail.com", role = c("aut"),
           comment = c(ORCID = "0000-0002-2974-1086")))
Description: High-throughput single-cell measurements of DNA methylation allows studying inter-cellular epigenetic heterogeneity, but this task faces the challenges of sparsity and noise. We present vmrseq, a statistical method that overcomes these challenges and identifies variably methylated regions accurately and robustly.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.2.3
biocViews: Software, ImmunoOncology, DNAMethylation, Epigenetics,
        SingleCell, Sequencing, WholeGenome
Depends: R (>= 4.5.0)
Imports: bumphunter, dplyr, BiocParallel, DelayedArray, GenomicRanges,
        ggplot2, methods, tidyr, locfit, gamlss.dist, recommenderlab,
        HDF5Array, data.table, SummarizedExperiment, IRanges,
        S4Vectors, devtools
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://github.com/nshen7/vmrseq
BugReports: https://github.com/nshen7/vmrseq/issues
git_url: https://git.bioconductor.org/packages/vmrseq
git_branch: devel
git_last_commit: f4be463
git_last_commit_date: 2026-08-06
Repository: Bioconductor 3.24
Date/Publication: 2026-08-09
NeedsCompilation: no
Packaged: 2026-08-10 00:58:29 UTC; biocbuild
Author: Keegan Korthauer [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4565-1654>),
  Ning Shen [aut] (ORCID: <https://orcid.org/0000-0002-2974-1086>)
Maintainer: Keegan Korthauer <kkorthauer@cmmt.ubc.ca>
