% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\docType{data}
\name{cell_1}
\alias{cell_1}
\title{cell_1}
\format{
A data frame with 10000 rows and 5 variables (no column names):
\describe{
  \item{chr}{Chromosome}
  \item{mc_count}{Genomic coordinate}
  \item{pos}{Strand information}
  \item{strand}{Number of methylated reads}
  \item{total}{Number of reads in total}
}
}
\usage{
data(cell_1)
}
\description{
This dataset is an example of a single-cell file that can be input to
\code{vmrseq::poolData} function. It contains the first 10000 CpG sites in
a random cell from mouse frontal cortext dataset published by Luo et al. (2017).
}
\examples{
data(cell_1)
head(cell_1)

}
\references{
Luo, Chongyuan et al. \emph{Single-cell methylomes identify neuronal
subtypes and regulatory elements in mammalian cortex.}. Science (New York, N.Y.)
vol. 357,6351 (2017): 600-604.
}
\keyword{datasets}
